% ELEMENTI DI TOPOGRAFIA E TRATTAMENTO DELLE OSSERVAZIONI
% seconda edizione
% Barzaghi Riccardo, Pinto Livio, Pagliari Diana
% ottobre 2018
% Script realized under BY NC Creative Commons License
% tested for Matlab 2016b

% 2.8: ESERCIZIO DI INQUADRAMENTO DI UN RILEVAMENTO PLANIMETRICO

%% DATI 
clc, clear all, close all

%coordinate dei punti 1-5 nel sistema di riferimento locale
v_loc=[1 373.148 -454.968;
       2 486.221  -72.811;
       3 542.758  379.126;
       4 -265.384 385.772;
       5 -654.49  -89.429];

%coordinate dei punti PF1 e PF2 nel sistema di riferimento locale
PF_loc={'PF1' 291.517 -678.801;
        'PF2' -1255.941 553.596};

%coordinate dei punti PF1 e PF2 nel sistema di riferimento catastale
PF_cat={'PF1' 12557.12 7514.47;
        'PF2' 10627.76 7954.47};

    %% CALCOLO DELLA SOLUZIONE
% La trasformazione tra i due sdr  una rototraslazione con fattore di
% scala --> stima di 4 parametri (traslazione E0 e N0, fattore di scala lambda, angolo di rotazione theta)
% La rototraslazione  descritta dalla equazione X=X0+lambda*R*x
% trattandosi di un problema non lineare  necessario linearizzarlo
% ponendo a=lambda*cos(theha) b=lambda*sin(theha)
   
% Y = Ax + b
%[EPF1     [1 0 xPF1 -yPF1    [E0
% NPF1      0 1 yPF1  xPF1  *  N0
% EPF2   =  1 0 xPF2 -yPF2     a
% NPF2]     0 1 yPF2  xPF2]    b ]
    
% POPOLAMENTO MATRICI 
% A=matrice deli coefficienti delle incognite
    
A=[1 0 PF_loc{1,2} -PF_loc{1,3};
   0 1 PF_loc{1,3}  PF_loc{1,2}
   1 0 PF_loc{2,2} -PF_loc{2,3};
   0 1 PF_loc{2,3}  PF_loc{2,2}];
   
% vettore delle quantit osservate
Y=[PF_cat{1,2};
   PF_cat{1,3};
   PF_cat{2,2};
   PF_cat{2,3}];
   
% vettore dei termini noti
 b=zeros(4,1);
  
% calcolo del vettore delle incognite x 
 x=A\(Y-b);
   
%stima dell'angolo di rotazione theta 
 theta = atan2(x(4),x(3)); %rad
 theta_g= theta*200/pi;

% stima del fattore di scala lambda
 lambda=sqrt(x(3)^2+x(4)^2);
  
  
      %% CALCOLO DELLE COORDINATE E N DEI VERTICI 1-5
  % YY = AAx + bb
  %[Eloc1     [1 0 xloc1 -yloc1    
  % Nloc1      0 1 yloc1  xloc1  
  % Eloc2   =  1 0 xloc2 -yloc2    
  % Nloc2      0 1 yloc2  xloc2    
  % Eloc3      1 0 xloc3 -yloc3  * [E0
  % Eloc3      0 1 yloc3  xloc3     N0
  % Eloc4      1 0 xloc4 -yloc4     a
  % Eloc4      0 1 yloc4  xloc4     b]
  % Eloc5      1 0 xloc5 -yloc5
  % Eloc5      0 1 yloc5  xloc5]
    
    
 % POPOLAMENTO MATRICI 
 % AA=matrice deli coefficienti delle incognite
    
    AA=[1 0 v_loc(1,2) -v_loc(1,3);
        0 1 v_loc(1,3)  v_loc(1,2);
        1 0 v_loc(2,2) -v_loc(2,3);
        0 1 v_loc(2,3)  v_loc(2,2);
        1 0 v_loc(3,2) -v_loc(3,3);
        0 1 v_loc(3,3)  v_loc(3,2);
        1 0 v_loc(4,2) -v_loc(4,3);
        0 1 v_loc(4,3)  v_loc(4,2);
        1 0 v_loc(5,2) -v_loc(5,3);
        0 1 v_loc(5,3)  v_loc(5,2)];
  
 %vettore dei termini noti  
  bb=zeros(size(AA,1),1);
  
 % calcolo delle coordinate dei vertici 1-5
  YY=AA*x+bb;
  v_cat=[1 YY(1) YY(2)
         2 YY(3) YY(4)
         3 YY(5) YY(6)
         4 YY(7) YY(8)
         5 YY(9) YY(10)];
     
         
  
      %% GRAFICI
 
%grafico nel sdr locale dei vertici 1-5 e dei punti catastali PF1 e PF2
figure (1);
hold on 
axis equal
step=20;
plot(v_loc(:,2),v_loc(:,3),'+b','MarkerSize', 10);
text(v_loc(:,2)+step,v_loc(:,3)+step,num2str(v_loc(:,1)));
plot(PF_loc{1,2},PF_loc{1,3},'^r','MarkerSize', 6, 'MarkerFaceColor','r');
text(PF_loc{1,2}+step,PF_loc{1,3}+step,PF_loc{1,1});
plot(PF_loc{2,2},PF_loc{2,3},'^r','MarkerSize', 6, 'MarkerFaceColor','r');
text(PF_loc{2,2}+step,PF_loc{2,3}+step,PF_loc{2,1});
xlabel('x[m]');
ylabel('y[m]');
title('sistema di riferimento locale');
legend('vertici 1-5','Punti Fiduciali','Location','southoutside', 'Orientation', 'horizontal');

%grafico nel sdr NE dei vertici 1-5 e dei punti catastali PF1 e PF2
figure (2);
hold on 
axis equal
step=20;
plot(v_cat(:,2),v_cat(:,3),'+b','MarkerSize', 10);
text(v_cat(:,2)+step,v_cat(:,3)+step,num2str(v_cat(:,1)));
plot(PF_cat{1,2},PF_cat{1,3},'^r','MarkerSize', 6, 'MarkerFaceColor','r');
text(PF_cat{1,2}+step,PF_cat{1,3}+step,PF_cat{1,1});
plot(PF_cat{2,2},PF_cat{2,3},'^r','MarkerSize', 6, 'MarkerFaceColor','r');
text(PF_cat{2,2}+step,PF_cat{2,3}+step,PF_cat{2,1});
xlabel('E[m]');
ylabel('N[m]');
title('sistema di riferimento dei PF');
legend('vertici 1-5','Punti Fiduciali','Location','southoutside','Orientation', 'horizontal');