% ELEMENTI DI TOPOGRAFIA E TRATTAMENTO DELLE OSSERVAZIONI
% seconda edizione
% Barzaghi Riccardo, Pinto Livio, Pagliari Diana
% ottobre 2018
% Script realized under BY NC Creative Commons License
% tested for Matlab 2016b

% ESERCIZIO 2.2: TRASFORMAZIONI TRA SISTEMI DI RIFERIMENTO 2D


%% DATI 
clc, clear all, close all

%coordinate dei punti 1-4 nel sistema di riferimento locale (coordinate
%polari)
%nome pt theta[gon] d[m]
p_loc_pol=[1  47.4678  139.357;
           2 112.3112  122.994;
           3 254.9812  111.722;
           4 326.8115  120.892];

%coordinate dei punti PF1,PF2, PF2 nel sistema di riferimento locale
%(coordinate polari)
PF_loc_pol={'PF1' 378.4489 224.1291;
            'PF2'  56.4189 219.333
            'PF3' 235.5891 290.005};

%coordinate dei punti PF1,PF2, PF2 nel sistema di riferimento catastale
PF_cat={'PF1' 12467.367 7410.543;
        'PF2' 12558.763 7172.660
        'PF3' 12056.774 7194.599};

        %% CALCOLO DELLE COORDINATE NEL SISTEMA DI RIFERIMENTO LOCALE
% TRASFORMAZIONE DA COORDINATE POLARI A COORDINATE CARTESIANE LOCALI
%coordinate dei punti 1-4 nel sistema di riferimento locale
%coordinate cartesiane locali
p_loc_cart= [(1:4)' p_loc_pol(:,3).*sin(p_loc_pol(:,2)/200*pi),p_loc_pol(:,3).*cos(p_loc_pol(:,2)/200*pi)];
        
%coordinate dei punti PF1,PF2, PF2 nel sistema di riferimento locale
%coordinate cartesiane locali
PF_loc_cart={'PF1' PF_loc_pol{1,3}*sin(PF_loc_pol{1,2}/200*pi) PF_loc_pol{1,3}*cos(PF_loc_pol{1,2}/200*pi);
          'PF2' PF_loc_pol{2,3}*sin(PF_loc_pol{2,2}/200*pi) PF_loc_pol{2,3}*cos(PF_loc_pol{2,2}/200*pi);
          'PF3' PF_loc_pol{3,3}*sin(PF_loc_pol{3,2}/200*pi) PF_loc_pol{3,3}*cos(PF_loc_pol{3,2}/200*pi);};


          %% STIMA DEI PARAMETRI DELLA TRASFORMAZIONE TRA I DUE SDR
    %% CALCOLO DELLA SOLUZIONE
% La trasformazione tra i due sdr  una rototraslazione con fattore di
% scala --> stima di 4 parametri (traslazione E0 e N0, fattore di scala lambda, angolo di rotazione theta)
% La rototraslazione  descritta dalla equazione X=X0+lambda*R*x
% trattandosi di un problema non lineare  necessario linearizzarlo
% ponendo a=lambda*cos(theha) b=lambda*sin(theha)
   
% Y = Ax + b
%[EPF1     [1 0 xPF1 -yPF1    [E0
% NPF1      0 1 yPF1  xPF1  *  N0
% EPF2   =  1 0 xPF2 -yPF2     a
% NPF2]     0 1 yPF2  xPF2]    b ]
    
% POPOLAMENTO MATRICI 
% A=matrice deli coefficienti delle incognite
    
A=[1 0 PF_loc_cart{1,2} -PF_loc_cart{1,3};
   0 1 PF_loc_cart{1,3}  PF_loc_cart{1,2}
   1 0 PF_loc_cart{2,2} -PF_loc_cart{2,3};
   0 1 PF_loc_cart{2,3}  PF_loc_cart{2,2}];
   
% vettore delle quantit osservate
Y=[PF_cat{1,2};
   PF_cat{1,3};
   PF_cat{2,2};
   PF_cat{2,3}];
   
% vettore dei termini noti
 b=zeros(4,1);
  
% calcolo del vettore delle incognite x 
 x=A\(Y-b);
   
%stima dell'angolo di rotazione theta 
 theta = atan2(x(4),x(3)); %rad
 theta_g= theta*200/pi;

% stima del fattore di scala lambda
 lambda=sqrt(x(3)^2+x(4)^2);
  
      %% CALCOLO DELLE COORDINATE E N DEI VERTICI 1-5
  % YY = AAx + bb
  %[Eloc1     [1 0 xloc1 -yloc1    
  % Nloc1      0 1 yloc1  xloc1  
  % Eloc2   =  1 0 xloc2 -yloc2    
  % Nloc2      0 1 yloc2  xloc2    
  % Eloc3      1 0 xloc3 -yloc3  * [E0
  % Eloc3      0 1 yloc3  xloc3     N0
  % Eloc4      1 0 xloc4 -yloc4     a
  % Eloc4      0 1 yloc4  xloc4]     b]
    
    
 % POPOLAMENTO MATRICI 
 % AA=matrice deli coefficienti delle incognite
    
    AA=[1 0 p_loc_cart(1,2)  -p_loc_cart(1,3);
        0 1 p_loc_cart(1,3)   p_loc_cart(1,2);
        1 0 p_loc_cart(2,2)  -p_loc_cart(2,3);
        0 1 p_loc_cart(2,3)   p_loc_cart(2,2);
        1 0 p_loc_cart(3,2)  -p_loc_cart(3,3);
        0 1 p_loc_cart(3,3)   p_loc_cart(3,2);
        1 0 p_loc_cart(4,2)  -p_loc_cart(4,3);
        0 1 p_loc_cart(4,3)   p_loc_cart(4,2)];
  
 %vettore dei termini noti  
  bb=zeros(size(AA,1),1);
  
 % calcolo delle coordinate dei vertici 1-5
  YY=AA*x+bb;
  v_cat=[1 YY(1) YY(2)
         2 YY(3) YY(4)
         3 YY(5) YY(6)
         4 YY(7) YY(8)];
     
         
       %% GRAFICI
 
%grafico nel sdr locale dei vertici 1-5 e dei punti catastali PF1 e PF2
figure (1);
hold on 
axis equal
step=20;
plot(p_loc_cart(:,2),p_loc_cart(:,3),'+b','MarkerSize', 10);
text(p_loc_cart(:,2)+step,p_loc_cart(:,3)+step,num2str(p_loc_cart(:,1)));
plot(PF_loc_cart{1,2},PF_loc_cart{1,3},'^r','MarkerSize', 6, 'MarkerFaceColor','r');
text(PF_loc_cart{1,2}+step,PF_loc_cart{1,3}+step,PF_loc_cart{1,1});
plot(PF_loc_cart{2,2},PF_loc_cart{2,3},'^r','MarkerSize', 6, 'MarkerFaceColor','r');
text(PF_loc_cart{2,2}+step,PF_loc_cart{2,3}+step,PF_loc_cart{2,1});
xlabel('x[m]');
ylabel('y[m]');
title('sistema di riferimento locale');
legend('vertici 1-5','Punti Fiduciali','Location','southoutside', 'Orientation', 'horizontal');

%grafico nel sdr NE dei vertici 1-5 e dei punti catastali PF1 e PF2
figure (2);
hold on 
axis equal
step=20;
plot(v_cat(:,2),v_cat(:,3),'+b','MarkerSize', 10);
text(v_cat(:,2)+step,v_cat(:,3)+step,num2str(v_cat(:,1)));
plot(PF_cat{1,2},PF_cat{1,3},'^r','MarkerSize', 6, 'MarkerFaceColor','r');
text(PF_cat{1,2}+step,PF_cat{1,3}+step,PF_cat{1,1});
plot(PF_cat{2,2},PF_cat{2,3},'^r','MarkerSize', 6, 'MarkerFaceColor','r');
text(PF_cat{2,2}+step,PF_cat{2,3}+step,PF_cat{2,1});
xlabel('E[m]');
ylabel('N[m]');
title('sistema di riferimento dei PF');
legend('vertici 1-5','Punti Fiduciali','Location','southoutside','Orientation', 'horizontal');
