% ELEMENTI DI TOPOGRAFIA E TRATTAMENTO DELLE OSSERVAZIONI
% seconda edizione
% Barzaghi Riccardo, Pinto Livio, Pagliari Diana
% ottobre 2018
% Script realized under BY NC Creative Commons License
% tested for Matlab 2016b

%ESERCIZIO 4.2: CALCOLO DELLE COORDINATE GEOGRAFICHE DI UN PUNTO Q

%Si risolve il precedente Esercizio 1.1 svolgendo i calcoli sul piano di
%Gauss
% Al termine dello script sono due funzioni che implementano le formule di
% Hirvonen (le funzioni sono state inserite come funzioni locali nello
% script, ma possono essere copiate e richiamate esternamente anche da
% script differenti realizzati dallo studente)

%% DATI
clc, clear all, close all

a=6378388;
e_2=0.00672267;
lam_fuso=9;
f_o=1500000;
lambdaP=[9 13 39];
phiP=[45 28 40];
hP=163.00; %m


%thetaPQ=27.2552
alpha=(27.2552)/180*pi; %rad
%zPQ=83.5626
xi=(83.5626)/180*pi; %rad
D= 3001.342; %m

%% SVOLGIMENTO
% conversione delle coordinate geografiche del punto P in coordinate
% cartografiche utilizzando le formule di Hirvonen
[EP,NP]=philam2EN(a,e_2,lam_fuso, f_o, lambdaP, phiP);

%conversione phiP e lamdaP in radianti
lambdaP_r=(lambdaP(1)+lambdaP(2)/60+lambdaP(3)/3600)*pi/180;
phiP_r=(phiP(1)+phiP(2)/60+phiP(3)/3600)*pi/180;

%calcolo delle coordinate depurate della falsa origine
f_or=1500000;
EP_d=EP-f_or; %m

%calcolo del raggio RP
RP=((a/(1-e_2*(sin(phiP_r))^2)^0.5)*((a*(1-e_2))/(1-e_2*(sin(phiP_r))^2)^1.5))^0.5; %m

%calcolo del modulto di deformazione lineare
m=0.9996*(1+(EP_d^2)/(0.9996^2*2*RP^2));

delta=atan((D*sin(xi))/(hP+RP+D*cos(xi)));

% approssimazione sferica
s=RP*delta;
dPQ=s*m; %m

%calcolo della convergenza del meridiano
gamma=(lambdaP_r-9/180*pi)*sin(phiP_r);

%calcolo dell'angolo alpha sulla carta
alpha_cart=alpha-abs(gamma);

%% CALCOLO DELLE COORDINATE DEL PUNTO Q
EQ=EP+dPQ*sin(alpha_cart);
NQ=NP+dPQ*cos(alpha_cart);

% conversione delle coordinate cartografiche del punto Q in coordinate
% geografiche utilizzando le formule di Hirvonen
[lambdaQ,phiQ]=NE2philamb(a,e_2,EQ,NQ,f_o);

%% FORMULE HIRVOVEN

%formula di Hirvonen per la conversione da coordinate geografiche a
%coordinate cartografiche
function [E,N]=philam2EN(a,e_2,lam_fuso, f_o, lambda, phi)
%INPUT
% a e e_2 parametri ellissoidici a ed e^2
% lam_fuso: longitudine del fuso centrale
% f_o: falsa origine Est
% lambda: longitudine del punto da trasformare
% phi: latitudine del punto da trasformare
%OUTPUT
% E coordinata cartografica Est
% N coordinate cartografica N

%es. per richiamare la funzione
%[E,N]=philam2EN(a,e_2,lam_fuso, f_o, lambda, phi)

Cr=0.9996;
e_2_p=e_2/(1-e_2);
c=Cr*a/(1-e_2)^0.5;
A1=a*Cr*(1-e_2/4-3*e_2^2/64-5*e_2^3/256);
A2=a*Cr*(3*e_2/8+3*e_2^2/32+45*e_2^3/1024);
A4=a*Cr*(15*e_2^2/256+45*e_2^3/1024);
A6=a*Cr*35*e_2^3/3072;

lambda_r=(lambda(1)+lambda(2)/60+lambda(3)/3600-lam_fuso)*pi/180;
phi_r=(phi(1)+phi(2)/60+phi(3)/3600)*pi/180;

v1=(1+((e_2_p)*(cos(phi_r))^2))^0.5;
x=atan((tan(phi_r))/(cos(v1*lambda_r)));
v=(1+((e_2_p)*(cos(x))^2))^0.5;

N=(A1*x)-(A2*sin(2*x))+(A4*sin(4*x))-(A6*sin(6*x));
E=f_o+c*asinh((cos(x)*tan(lambda_r))/v);
end

%formula di Hirvonen per la conversione da coordinate cartografiche a
%coordinate geografiche 
function [phi,lambda]=NE2philamb(a,e_2,E,N,f_O)
%INPUT
% a e e_2 parametri ellissoidici a ed e^2
% E coordinata cartografica Est
% N coordinate cartografica N
%OUTPUT
% phi: latitudine del punto da trasformare
% lambda: longitudine del punto da trasformare
Cr=0.9996;
e_2_p=e_2/(1-e_2);
e1=(1-(1-e_2)^0.5)/(1+(1-e_2)^0.5);
c=a/(1-e_2)^0.5;
B1=(3*e1/2)-(27*(e1^3)/32);
B2=(21*(e1^2)/16)-(55*(e1^4)/32);
B3=(151*(e1^3)/96);
B4=(1097*(e1^4)/512);
A1=a*(1-e_2/4-3*e_2^2/64-5*e_2^3/256);

theta=(N/Cr)/A1;
csi=theta+B1*sin(2*theta)+B2*sin(4*theta)+B3*sin(6*theta)+B4*sin(8*theta);
v1=(1+e_2_p*(cos(csi))^2)^0.5;

E_r=(E-f_O)/Cr;
long=atan((v1*sinh(E_r/c))/(cos(csi)));
lat=atan(tan(csi)*cos(v1*long));

long_dec=long/pi*180;
lat_dec=lat/pi*180;

phi=[floor(lat_dec) floor((lat_dec-floor(lat_dec))*60) ((lat_dec-floor(lat_dec))*60-floor((lat_dec-floor(lat_dec))*60))*60];
lambda=[floor(long_dec) floor((long_dec-floor(long_dec))*60) ((long_dec-floor(long_dec))*60-floor((long_dec-floor(long_dec))*60))*60];


end