% ELEMENTI DI TOPOGRAFIA E TRATTAMENTO DELLE OSSERVAZIONI
% seconda edizione
% Barzaghi Riccardo, Pinto Livio, Pagliari Diana
% ottobre 2018
% Script realized under BY NC Creative Commons License
% tested for Matlab 2016b

%ESEMPIO 6.7:
% STIMA AI MINIMI QUADRATI DEI PARAMETRI DI UNA FUNZIONE ESPONENZIALE
% y=Ke^(-bt)

%% DATI
clc, clear all, close all

tt=[2 4 6 8 10 12]';
yy=[2.0120 1.3460 0.9006 0.6077 0.4070 0.2682]';

%% CALCOLO DEI VALORI APPROSSIMATI
% per il calcolo dei valori approssimati si utilizzano le osservaazioni
% relative ai tempi t=2 e t=12
syms k b y t
S=solve('k*exp(-2*b)=2.0120','k*exp(-12*b)=0.2682','Real', true);
k_app=double(S.k);
b_app=double(S.b);

% I valori approssimati calcolati per via numerica tramite Matlab sono
% differenti rispetto a quelli riportati nel testo
% questo implica che i valori delle correzioni calcolate saranno diversi,
% ma il valore finale dei parametri stimati sar il medesimo 

%% LINEARIZZAZIONE
y=k*exp(-b*t);
dy=@(k, b, t) y;

d_k=diff(dy,k);
d_b=diff(dy,b);

%% POPOLAMENTO MATRICI
alpha0=yy;
L=k_app*exp(-b_app*tt);

A=zeros(size(tt,1),2);
for i=1:size(tt,1)
A(i,1)=double(subs(d_k,[k,b,t],[k_app,b_app,tt(i)]));
A(i,2)=double(subs(d_b,[k,b,t],[k_app,b_app,tt(i)]));
end

%% CALCOLO DELLE SOLUZIONI - 1A ITERAZIONE
delta=alpha0-L;
deltax=inv(A'*A)*A'*delta;
x=[k_app, b_app]'+deltax;

% Si lascia allo studente l'implemetazione della seconda iterazione, che
% sar caratterizzata da correzioni sempre pi piccole


