% ELEMENTI DI TOPOGRAFIA E TRATTAMENTO DELLE OSSERVAZIONI
% seconda edizione
% Barzaghi Riccardo, Pinto Livio, Pagliari Diana
% ottobre 2018
% Script realized under BY NC Creative Commons License
% tested for Matlab 2016b

%ESERCIZIO 6.4: COMPENSAZIONE DI UNA RETE DI LIVELLAZIONE GEOMETRICA

%% DATI
clc, clear all, close all

%punto_indietro/punto_avanti/dislivello[m]/distanza[m]/sigma[m]
data={1 2  -0.6715 1500 0.001225
     1 'A'  1.2336 1000 0.001
     1 'B' -0.8930 1000 0.001
     2 'A'  1.9021 1000 0.001
     2 'B' -0.2215 1000 0.001};
 
h_A=3.451;
h_B=1.326;

%% POPOLAMENTO MATRICI MQ
%vettore delle incognite del problema
incognite=[1 2];
%funzione per scrivere la matrice disegno A
A=riempiA_alt(data,incognite);
%vettore delle osservazioni alpha_0
alpha_0=cell2mat(data(:,3));
%vettore quote punti noti
h=[h_A h_B];
%vettore punti noti (stesso ordine utilizzato nel vettore h)
pnt_noti= {'A', 'B'};
% vettore dei termini noti L
L=riempiL_alt(data, pnt_noti, h);
%matrice dei pesi P


sigma_dz=0.001; %m
sigma02Priori=sigma_dz^2;

P=diag(sigma02Priori./cell2mat(data(:,5)).^2);


%% SOLUZIONE MQ
%matrice normale N
N=A'*P*A;
%termine noto Tn
Tn=A'*P*(alpha_0-L);
% vettore delle incognite x
x=N\Tn;
% x=N\Tn  un metodo pi efficace e stabile in matlab per calcolare
% x=inv(N)*Tn. E' utile sopratutto nel caso di grosse matrici
%vettore delle osservazioni teoriche
alpha=A*x+L;
%vettore degli scarti u
u=alpha_0-A*x-L;
%sigma0^2 posteriori
sigma_02_post=(u'*P*u)/(size(A,1)-size(A,2));
%matrice di varianza e covarianza delle incognite Cxx
Cxx=sigma_02_post*inv(N);
sigmaQ1=sqrt(Cxx(1,1));
sigmaQ2=sqrt(Cxx(2,2));
%matrice di varianza e covarianza delle osservazioni C_alpha_alpha
C_alpha_alpha=sigma_02_post*A*inv(N)*A';
%matrice varianza e covarianza degli scarti
C_uu=sigma_02_post*inv(P)-C_alpha_alpha;


%% TEST STATISTICI

% TEST CHI QUADRO RETE ALTRIMETRICA
val_Alt=sigma_02_post/sigma02Priori*(size(A,1)-size(A,2));
chi2_Alt = [chi2inv(0.025,(size(A,1)-size(A,2))),chi2inv(0.975,(size(A,1)-size(A,2)))];
if val_Alt>chi2_Alt(1)&& val_Alt<chi2_Alt(2)
   fprintf('Compensazione Altimetrica: Test Globale superato\n'); 
else
   fprintf('Compensazione Altimetrica: Test Globale NON superato\n'); 
end

% TEST RESIDUI STANDARDIZZATI RETE ALTIMETRICA

%test residui =[residuo standardizzato,flag]
%flag=1 --> test non superato
%flag=0 --> test superato

test_residui=u./sqrt(diag(C_uu));

norm_dist = norminv([0.025 0.975],0,1);

ind= test_residui(:,1)<min(norm_dist)|test_residui(:,1)>max(norm_dist);
test_residui(ind,2)=1;
clear ind i

if any(test_residui(:,2))
   fprintf('Compensazione Altimetrica: Test dei resuidui standardizzati NON superato\n');
else
    fprintf('Compensazione Altimetrica: Test dei resuidui standardizzati superato\n');

end

%% FUNZIONI

function [A]=riempiA_alt(data,incognite)

% funzione che scrive la matrice disegno A
% nel caso di una rete altimetrica
% INPUT
% data: matrice delle osservazioni organizzata come
%   punto_indietro/punto_avanti/dislivello[m]/distanza[m]/sigma[m]
% incognite: vettore delle incognite
% OUTPUT
% matrice disegno A


A=zeros(size(data,1),length(incognite));
for i=1:length(incognite)
    idx1= [data{:,1}]==incognite(1,i);
    A(idx1,i)=-1;
    idx2= [data{:,2}]==incognite(1,i);
    A(idx2,i)=1;
end

end

function [L]=riempiL_alt(data,t_noti, h)

% funzione che scrive il vettore dei termini noti L
% nel caso di una rete altimetrica
% INPUT
% data: matrice delle osservazioni organizzata come
%   punto_indietro/punto_avanti/dislivello[m]/distanza[m]/sigma[m]
% t_noti: cell array contenete i nomi dei punti noti
% h: vettore contenente le quote dei punti noti (stesso ordine usato in
% t_noti)
% OUTPUT
% vettore dei termini noti L

L=zeros(size(data,1),1);
for i=1:length(t_noti)
     idx1 = strcmp(data(:,1), t_noti{1,i});
     L(idx1,1)=-h(1,i);
     idx2 = strcmp(data(:,2), t_noti{1,i});
     L(idx2,1)=h(1,i);

end


end

% 

